import scipy.io
import numpy as np
import os
import sys

## The first argument is the name of the input file (which should be in the same folder as this script)
## The second argument is the path to the data directory (in the script classifyRelations/tempData) where
## the output files will be written (rawSentences.txt and toBeConverted.mat)

def createCatDic(catNames):
    catDic = {}
    label = 0;
    for cat in catNames:
        catDic[catNames[label]]=label+1
        label += 1
    return catDic


def saveData(elemStrings,elemInd,sentenceLabels,catNames,dataDir):
    elemStringsCell = np.zeros((len(elemStrings),2), dtype=np.object)
    i=0
    for pair in elemStrings:
        elemStringsCell[i] = pair
        i += 1
    catCell = np.zeros((1,len(catNames)), dtype=np.object)
    i = 0
    for name in catNames:
        catCell[0,i] = name
        i += 1
    data = {}
    data['elemInd'] = elemStringsCell
    data['numInd'] = elemInd
    data['categories'] = catCell
    data['sentenceLabels'] = sentenceLabels
    scipy.io.savemat(dataDir + '/toBeConverted.mat',data,oned_as='column')



if __name__ == '__main__':

    #input sentences
    sentences = open(sys.argv[1], "r")
    dataDir = sys.argv[2]
    
    # temporary output for the stanford parser    
    rawSentencesStr = dataDir + '/rawSentences.txt'    
    rawSentences = open(rawSentencesStr, "w")

    elemStrings = [] # 
    elemInd = [] # the index of the elements
    numberWritten = 0
    sentenceNum = 1
    
    catNames = ['Cause-Effect(e1,e2)','Component-Whole(e1,e2)','Content-Container(e1,e2)','Entity-Destination(e1,e2)','Entity-Origin(e1,e2)',
                'Instrument-Agency(e1,e2)','Member-Collection(e1,e2)','Message-Topic(e1,e2)','Product-Producer(e1,e2)','Other',
                'Cause-Effect(e2,e1)','Component-Whole(e2,e1)','Content-Container(e2,e1)','Entity-Destination(e2,e1)','Entity-Origin(e2,e1)',
                'Instrument-Agency(e2,e1)','Member-Collection(e2,e1)','Message-Topic(e2,e1)','Product-Producer(e2,e1)']
    catDic = createCatDic(catNames)

    sentenceLabel = []
    entLabel = []
    simScore = []
    
    #reading the topmost line of input which is not needed "sentence_A   sentence_B..."
    line = sentences.readline()

    #in each iteration of the loop we process one line, which means we are adding two sentences to rawSentences.txt
    for line in sentences:
	#parse line and break up into smaller pieces - write resulting sentences and put labels in an array
	
	temp = line.find("\t")
	line = line[temp+1:]
	temp = line.find("\t")
	sent1 = line[:temp]
	#sentences in rawSentences.txt must be terminated by a full stop
	sent1 = sent1 + ".\n"		

	line = line[temp+1:]
	temp = line.find("\t")
	sent2 = line[:temp]
	sent2 = sent2 + ".\n"
	
	line = line[temp+1:]
	temp = line.find("\t")
	scoreStr = line[:temp]
	score = float(scoreStr)
	
	line = line[temp+1:]
	temp = line.find("\t")
	labelStr = line[:temp]
	if labelStr == "NEUTRAL":
	    simLabel = 0
	elif labelStr == "ENTAILMENT":
	    simLabel = 1
	elif labelStr == "CONTRADICTION":
	    simLabel = 2
	else:
	    print "Error - invalid sentence entailment specification"	

	#write the two sentences into rawSentences.txt
	rawSentences.writelines(sent1)
	rawSentences.writelines(sent2)

	#add the labels to the required array
	simScore.append(score)
	entLabel.append(simLabel)


	elemStrings.append(["1","1"])
        elemInd.append([1,1])
	sentenceLabel.append(1)
        
	elemStrings.append(["1","1"])
        elemInd.append([1,1])
	sentenceLabel.append(1)

	numberWritten += 2
        sentenceNum += 2


    

    print "A total of " + str(numberWritten) + " sentences were found." 
    sentences.close()
    rawSentences.close()
    #runStanfordParser(stanfordParserDir, rawSentencesStr)
    #saveData(elemStrings,elemInd,sentenceLabel,catNames,dataDir)
    
    #add code here to save the two arrays simScore and entLabel in .mat files
    data = {}
    data['entail'] = entLabel
    data['similarity'] = simScore
    scipy.io.savemat('goldLabel.mat',data,oned_as='column')

    problemWords = ['programme','neighbour','clamour','organisation','harbour','labour','behaviour']
    replaceMents = ['program', 'neighbor','clamor','organization','harbor','labor','behavior']                    
    
